@@ -2,7 +2,7 @@ from typing import Optional
 
 from itertools import chain
 from AnyQt.QtCore import Qt, QAbstractTableModel, QSize, QItemSelectionModel, \
-                         QItemSelection
+    QItemSelection, QModelIndex
 from AnyQt.QtWidgets import QSizePolicy, QApplication, QTableView, \
     QStyledItemDelegate
 from AnyQt.QtGui import QColor
@@ -101,10 +101,8 @@ class ConcordanceModel(QAbstractTableModel):
     def flags(self, _):
         return Qt.ItemIsEnabled | Qt.ItemIsSelectable
 
-    def rowCount(self, parent=None, *args, **kwargs):
-        return 0 if parent is None or parent.isValid() or \
-                    self.word_index is None \
-               else len(self.word_index)
+    def rowCount(self, parent=QModelIndex(), *args, **kwargs):
+        return 0 if parent.isValid() or self.word_index is None else len(self.word_index)
 
     def columnCount(self, parent=None, *args, **kwargs):
         return 3
@@ -158,7 +156,7 @@ class OWConcordance(OWWidget):
     name = "Concordance"
     description = "Display the context of the word."
     icon = "icons/Concordance.svg"
-    priority = 30000
+    priority = 520
 
     class Inputs:
         corpus = Input("Corpus", Corpus)
@@ -173,7 +171,7 @@ class OWConcordance(OWWidget):
     autocommit = Setting(True)
     context_width = Setting(5)
     word = ContextSetting("", exclude_metas=False)
-    # TODO Set selection settings (DataHashContextHandler)
+    selected_rows = Setting([], schema_only=True)
 
     class Warning(OWWidget.Warning):
         multiple_words_on_input = Msg("Multiple query words on input. "
@@ -217,10 +215,7 @@ class OWConcordance(OWWidget):
         self.conc_view.setSelectionBehavior(QTableView.SelectRows)
         self.conc_view.setSelectionModel(DocumentSelectionModel(self.model))
         self.conc_view.setItemDelegate(HorizontalGridDelegate())
-        # connect selectionChanged to self.commit(), which will be
-        # updated by gui.auto_commit()
-        self.conc_view.selectionModel().selectionChanged.connect(lambda:
-                                                                 self.commit())
+        self.conc_view.selectionModel().selectionChanged.connect(self.selection_changed)
         self.conc_view.horizontalHeader().hide()
         self.conc_view.setShowGrid(False)
         self.mainArea.layout().addWidget(self.conc_view)
@@ -240,21 +235,39 @@ class OWConcordance(OWWidget):
             self.conc_view.selectionModel().select(sel,
                 QItemSelectionModel.SelectCurrent | QItemSelectionModel.Rows)
 
+    def selection_changed(self):
+        selection = self.conc_view.selectionModel().selection()
+        self.selected_rows = sorted(set(cell.row() for cell in selection.indexes()))
+        self.commit()
+
+    def set_selection(self, selection):
+        if selection:
+            sel = QItemSelection()
+            for row in selection:
+                index = self.conc_view.model().index(row, 0)
+                sel.select(index, index)
+            self.conc_view.selectionModel().select(sel,
+                QItemSelectionModel.SelectCurrent | QItemSelectionModel.Rows)
+
     @Inputs.corpus
     def set_corpus(self, data=None):
         self.closeContext()
         self.corpus = data
-        if data is not None and not isinstance(data, Corpus):
-            self.corpus = Corpus.from_table(data.domain, data)
-        self.model.set_corpus(self.corpus)
+        if data is None:    # data removed, clear selection
+            self.selected_rows = []
+
         if not self.is_word_on_input:
             self.word = ""
             self.openContext(self.corpus)
+
+        self.model.set_corpus(self.corpus)
         self.set_word()
 
     @Inputs.query_word
     def set_word_from_input(self, topic):
         self.Warning.multiple_words_on_input.clear()
+        if self.is_word_on_input:   # word changed, clear selection
+            self.selected_rows = []
         self.is_word_on_input = topic is not None and len(topic) > 0
         self.input.setEnabled(not self.is_word_on_input)
         if self.is_word_on_input:
@@ -268,6 +281,9 @@ class OWConcordance(OWWidget):
         self.update_widget()
         self.commit()
 
+    def handleNewSignals(self):
+        self.set_selection(self.selected_rows)
+
     def resize_columns(self):
         col_width = (self.conc_view.width() -
                      self.conc_view.columnWidth(1)) / 2 - 12
@@ -295,16 +311,25 @@ class OWConcordance(OWWidget):
             self.n_types = ''
 
     def commit(self):
-        rows = [sel_range.top() for sel_range
-                in self.conc_view.selectionModel().selection()]
         selected_docs = sorted(set(self.model.word_index[row][0]
-                                   for row in rows))
+                                   for row in self.selected_rows))
         if selected_docs:
             selected = self.corpus[selected_docs]
             self.Outputs.selected_documents.send(selected)
         else:
             self.Outputs.selected_documents.send(None)
 
+    def send_report(self):
+        view = self.conc_view
+        model = self.conc_view.model()
+        self.report_items("Concordances", (
+            ("Query", model.word),
+            ("Tokens", model.n_tokens),
+            ("Types", model.n_types),
+            ("Matching", self.n_matching),
+        ))
+        self.report_table(view)
+
 
 if __name__ == '__main__': # pragma: no cover
     app = QApplication([])
