@@ -12,8 +12,8 @@ from Orange.widgets.widget import OWWidget, Msg, Input, Output
 from orangecontrib.text import preprocess
 from orangecontrib.text.corpus import Corpus
 from orangecontrib.text.misc import nltk_data_dir
-from orangecontrib.text.tag import StanfordPOSTagger
-from orangecontrib.text.tag import taggers
+from orangecontrib.text.tag import StanfordPOSTagger, AveragedPerceptronTagger, \
+    MaxEntTagger
 from orangecontrib.text.widgets.utils import widgets, ResourceLoader
 from orangecontrib.text.widgets.utils.concurrent import asynchronous
 
@@ -66,7 +66,7 @@ class PreprocessorModule(gui.OWComponent, QWidget):
     def __init__(self, master):
         QWidget.__init__(self)
         gui.OWComponent.__init__(self, master)
-        self.master = master
+        self.master = master  # type: OWPreprocess
 
         # Title bar.
         title_holder = QWidget()
@@ -419,12 +419,28 @@ class FilteringModule(MultipleMethodModule):
             self.change_signal.emit()
 
     def read_stopwords_file(self, path):
-        self.methods[self.STOPWORDS].from_file(path)
+        self.master.Error.stopwords_encoding.clear()
+        self.master.Error.error_reading_stopwords.clear()
+        try:
+            self.methods[self.STOPWORDS].from_file(path)
+        except UnicodeError:
+            self.master.Error.stopwords_encoding()
+        except Exception as e:
+            self.master.Error.error_reading_stopwords(e)
+
         if self.STOPWORDS in self.checked:
             self.change_signal.emit()
 
     def read_lexicon_file(self, path):
-        self.methods[self.LEXICON].from_file(path)
+        self.master.Error.lexicon_encoding.clear()
+        self.master.Error.error_reading_lexicon.clear()
+        try:
+            self.methods[self.LEXICON].from_file(path)
+        except UnicodeError:
+            self.master.Error.lexicon_encoding()
+        except Exception as e:
+            self.master.Error.error_reading_lexicon(e)
+
         if self.LEXICON in self.checked:
             self.change_signal.emit()
 
@@ -453,14 +469,19 @@ class POSTaggingModule(SingleMethodModule):
     attribute = 'pos_tagger'
     enabled = settings.Setting(False)
 
-    STANFORD = len(taggers)
     stanford = settings.SettingProvider(ResourceLoader)
 
-    methods = taggers + [StanfordPOSTagger]
+    methods = [AveragedPerceptronTagger, MaxEntTagger, StanfordPOSTagger]
+    STANFORD = 2
+
     initialize_methods = False
 
     def setup_method_layout(self):
         super().setup_method_layout()
+        # initialize all methods except StanfordPOSTagger
+        # cannot be done in superclass due to StanfordPOSTagger
+        self.methods = [method() for method in self.methods[:self.STANFORD]]
+
         self.stanford = ResourceLoader(widget=self.master, model_format='Stanford model (*.model *.tagger)',
                                        provider_format='Java file (*.jar)',
                                        model_button_label='Model', provider_button_label='Tagger')
@@ -500,7 +521,7 @@ class OWPreprocess(OWWidget):
     name = 'Preprocess Text'
     description = 'Construct a text pre-processing pipeline.'
     icon = 'icons/TextPreprocess.svg'
-    priority = 30
+    priority = 200
 
     class Inputs:
         corpus = Input("Corpus", Corpus)
@@ -538,6 +559,10 @@ class OWPreprocess(OWWidget):
 
     class Error(OWWidget.Error):
         stanford_tagger = Msg("Problem while loading Stanford POS Tagger\n{}")
+        stopwords_encoding = Msg("Invalid stopwords file encoding. Please save the file as UTF-8 and try again.")
+        lexicon_encoding = Msg("Invalid lexicon file encoding. Please save the file as UTF-8 and try again.")
+        error_reading_stopwords = Msg("Error reading file: {}")
+        error_reading_lexicon = Msg("Error reading file: {}")
 
     class Warning(OWWidget.Warning):
         no_token_left = Msg('No tokens on output! Please, change configuration.')
